@@ -2,7 +2,6 @@ import numpy as N
 import ctypes
 assert ctypes.__version__ >= '1.0.1'
 import sys, os
-import pkg_resources
 THREAD_DEBUG = 0
 if THREAD_DEBUG:
     import threading
@@ -10,42 +9,56 @@ if THREAD_DEBUG:
 __all__ = ['CamIFaceError','BuffersOverflowed',
            'FrameTimeout',
            'FrameDataMissing',
+           'FrameDataCorrupt',
            'get_driver_name','get_num_cameras',
            'get_num_modes','get_mode_string',
            'get_camera_info','Camera',
            'get_wrapper_name','shutdown']
 
-backend_fname = os.environ.get('CAM_IFACE_CTYPES_BACKEND',None)
-if backend_fname is None:
-    if sys.platform.startswith('linux'):
-        #backend_fname = 'libcam_iface_camwire.so'
-        #backend_fname = 'libcam_iface_prosilica_gige.so'
-        #backend_fname = 'libcam_iface_dc1394.so'
-        backend_fname = 'libcam_iface_blank.so'
-    elif sys.platform.startswith('win'):
-        #backend_fname = 'cam_iface_imperx.dll'
-        backend_fname = 'cam_iface_blank.dll'
+if 1:
+    lib_name = 'mega'
+
+    if sys.platform == 'win32':
+        prefix = ''
+        extension = '.dll'
+    elif sys.platform.startswith('linux'):
+        prefix = 'lib'
+        extension = '.so'
     elif sys.platform.startswith('darwin'):
-        #backend_fname = 'libcam_iface_dc1394.dylib'
-        backend_fname = 'libcam_iface_blank.dylib'
+        prefix = 'lib'
+        extension = '.dylib'
     else:
         raise ValueError("unknown platform '%s'"%sys.platform)
 
-backend_fullpath = pkg_resources.resource_filename(__name__,backend_fname)
-if sys.platform.startswith('linux'):
-    if os.path.exists(backend_fullpath):
-        # Try to use version packaged with this module
-        c_cam_iface = ctypes.cdll.LoadLibrary(backend_fullpath)
-    else:
-        # Try to use system version
-        c_cam_iface = ctypes.cdll.LoadLibrary(backend_fname)
-elif sys.platform.startswith('win'):
-    c_cam_iface = ctypes.CDLL(backend_fullpath)
-elif sys.platform.startswith('darwin'):
-    c_cam_iface = ctypes.CDLL(backend_fullpath)
-else:
-    raise ValueError("unknown platform '%s'"%sys.platform)
+    backend_fname = (prefix+'cam_iface_'+
+                     lib_name+extension)
 
+backend_path = os.environ.get('CAM_IFACE_CTYPES_PATH',None)
+#backend_path=r"C:\Program Files\libcamiface 0.5.3\bin"
+if backend_path is not None:
+    if sys.platform=='win32':
+        SetDllDirectory = ctypes.windll.kernel32.SetDllDirectoryW
+        SetDllDirectory(backend_path)
+    else:
+        raise NotImplementedError('Unable to set backend search path')
+    print "set dll directory to '%s'. files:"%(backend_path,)
+    for f in os.listdir(backend_path):
+        print '  %s'%(f,)
+
+try:
+    if sys.platform.startswith('linux'):
+        # Try to use version packaged with this module
+        c_cam_iface = ctypes.cdll.LoadLibrary(backend_fname)
+    elif sys.platform.startswith('win'):
+        c_cam_iface = ctypes.CDLL(backend_fname)
+    elif sys.platform.startswith('darwin'):
+        c_cam_iface = ctypes.CDLL(backend_fname)
+    else:
+        raise ValueError("unknown platform '%s'"%sys.platform)
+except Exception,err:
+    print >> sys.stderr,'Could not open shared library for backend "%s"'%(
+        backend_fname,)
+    raise
 ##################################################
 
 # modified from numpy:
@@ -79,6 +92,10 @@ CAM_IFACE_RGB16S=9
 CAM_IFACE_RAW8=10
 CAM_IFACE_RAW16=11
 CAM_IFACE_ARGB8=12
+CAM_IFACE_MONO8_BAYER_BGGR=13
+CAM_IFACE_MONO8_BAYER_RGGB=14
+CAM_IFACE_MONO8_BAYER_GRBG=15
+CAM_IFACE_MONO8_BAYER_GBRG=16
 
 class Camwire_id(ctypes.Structure):
     _fields_ = [('vendor',ctypes.c_char*101),
@@ -107,19 +124,29 @@ class CameraPropertyInfo(ctypes.Structure):
                 ('scaled_unit_name',ctypes.c_char_p),
                 ('scale_offset',ctypes.c_double),
                 ('scale_gain',ctypes.c_double),
+                ('original_value',ctypes.c_long),
+                ('available',ctypes.c_int),
+                ('readout_capable',ctypes.c_int),
+                ('on_off_capable',ctypes.c_int),
+                ('absolute_capable',ctypes.c_int),
+                ('absolute_control_mode',ctypes.c_int),
+                ('absolute_min_value',ctypes.c_double),
+                ('absolute_max_value',ctypes.c_double),
                 ]
 
 c_cam_iface.cam_iface_get_api_version.restype = ctypes.c_char_p
 
 def _ensure_cam_iface_version_OK():
     actual = c_cam_iface.cam_iface_get_api_version()
-    expected = "20090331"
+    expected = "20091105"
     if actual != expected:
         raise RuntimeError("libcamiface mismatch: expected %s, got %s"%(
             expected,actual))
 
 _ensure_cam_iface_version_OK()
 
+c_cam_iface.cam_iface_have_error.restype = ctypes.c_int
+c_cam_iface.cam_iface_have_error.argtypes = []
 c_cam_iface.cam_iface_get_driver_name.restype = ctypes.c_char_p
 c_cam_iface.cam_iface_get_error_string.restype = ctypes.c_char_p
 c_cam_iface.cam_iface_get_num_modes.argtypes = [ctypes.c_int,
@@ -219,22 +246,31 @@ class FrameTimeout(CamIFaceError):
 class FrameDataMissing(CamIFaceError):
     pass
 
+class FrameDataCorrupt(CamIFaceError):
+    pass
+
 class FrameSystemCallInterruption(CamIFaceError):
     pass
 
+class NoFrameReturned(CamIFaceError):
+    pass
+
 class HardwareFeatureNotAvailable(CamIFaceError):
     pass
 
 def _check_error():
-    CAM_IFACE_BUFFER_OVERFLOW_ERROR=-392081
-    CAM_IFACE_FRAME_DATA_MISSING_ERROR=-392073
-    CAM_IFACE_FRAME_TIMEOUT=-392074
-    CAM_IFACE_FRAME_DATA_LOST_ERROR=-392075
-    CAM_IFACE_HARDWARE_FEATURE_NOT_AVAILABLE=-392076
-    CAM_IFACE_FRAME_INTERRUPTED_SYSCALL=-392078
-
     errnum = c_cam_iface.cam_iface_have_error()
     if errnum != 0:
+
+        CAM_IFACE_BUFFER_OVERFLOW_ERROR=-392081
+        CAM_IFACE_FRAME_DATA_MISSING_ERROR=-392073
+        CAM_IFACE_FRAME_TIMEOUT=-392074
+        CAM_IFACE_FRAME_DATA_LOST_ERROR=-392075
+        CAM_IFACE_HARDWARE_FEATURE_NOT_AVAILABLE=-392076
+        CAM_IFACE_FRAME_INTERRUPTED_SYSCALL=-392078
+        CAM_IFACE_SELECT_RETURNED_BUT_NO_FRAME_AVAILABLE=-392079
+        CAM_IFACE_FRAME_DATA_CORRUPT_ERROR=-392080
+
         err_str=c_cam_iface.cam_iface_get_error_string()
         if errnum == CAM_IFACE_BUFFER_OVERFLOW_ERROR:
             exc_type = BuffersOverflowed
@@ -242,10 +278,14 @@ def _check_error():
             exc_type = FrameDataMissing
         elif errnum == CAM_IFACE_FRAME_INTERRUPTED_SYSCALL:
             exc_type = FrameSystemCallInterruption
+        elif errnum == CAM_IFACE_SELECT_RETURNED_BUT_NO_FRAME_AVAILABLE:
+            exc_type = NoFrameReturned
         elif errnum == CAM_IFACE_FRAME_TIMEOUT:
             exc_type = FrameTimeout
         elif errnum == CAM_IFACE_HARDWARE_FEATURE_NOT_AVAILABLE:
             exc_type = HardwareFeatureNotAvailable
+        elif errnum == CAM_IFACE_FRAME_DATA_CORRUPT_ERROR:
+            exc_type = FrameDataCorrupt
         else:
             exc_type = CamIFaceError
         c_cam_iface.cam_iface_clear_error()
@@ -426,7 +466,16 @@ class Camera:
                   'max_value':info.max_value,
                   'has_auto_mode':info.has_auto_mode,
                   'has_manual_mode':info.has_manual_mode,
-                  'is_scaled_quantity':info.is_scaled_quantity}
+                  'is_scaled_quantity':info.is_scaled_quantity,
+                  'original_value':info.original_value,
+                  'available':info.available,
+                  'readout_capable':info.readout_capable,
+                  'on_off_capable':info.on_off_capable,
+                  'absolute_capable':info.absolute_capable,
+                  'absolute_control_mode':info.absolute_control_mode,
+                  'absolute_min_value':info.absolute_min_value,
+                  'absolute_max_value':info.absolute_max_value,
+                  }
         if result['is_scaled_quantity']:
             result.update({'scaled_unit_name':info.scaled_unit_name,
                            'scale_offset':info.scale_offset,
@@ -488,6 +537,14 @@ class Camera:
             return 'RGB8'
         elif self.cval.contents.coding == CAM_IFACE_ARGB8:
             return 'ARGB8'
+        elif self.cval.contents.coding == CAM_IFACE_MONO8_BAYER_BGGR:
+            return 'MONO8:BGGR'
+        elif self.cval.contents.coding == CAM_IFACE_MONO8_BAYER_RGGB:
+            return 'MONO8:RGGB'
+        elif self.cval.contents.coding == CAM_IFACE_MONO8_BAYER_GRBG:
+            return 'MONO8:GRBG'
+        elif self.cval.contents.coding == CAM_IFACE_MONO8_BAYER_GBRG:
+            return 'MONO8:GBRG'
         else:
             raise NotImplementedError("Can't convert pixel coding representation to string")
 
